<?php

namespace Drupal\trash\EntityQuery;

/**
 * Provides trash-specific helpers for altering entity queries.
 *
 * @property string $entityTypeId
 * @property \Drupal\Core\Entity\EntityTypeInterface $entityType
 * @property \Drupal\Core\Database\Query\ConditionInterface $condition
 */
trait QueryTrait {

  /**
   * Ensures that the query returns only deleted entities.
   *
   * @return $this
   */
  public function isDeleted(): static {
    $this->addMetaData('trash', 'inactive');
    return $this;
  }

  /**
   * Ensures that the query returns only non-deleted entities.
   *
   * @return $this
   */
  public function isNotDeleted(): static {
    $this->addMetaData('trash', 'active');
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareTrash(): static {
    /** @var \Drupal\trash\TrashManagerInterface $trash_manager */
    $trash_manager = \Drupal::service('trash.manager');
    if (!$trash_manager->shouldAlterQueries() || !$trash_manager->isEntityTypeEnabled($this->entityType)) {
      return $this;
    }

    // Skip altering queries with an explicit filter on the 'deleted' field, so
    // you can still list deleted content, if needed.
    $deleted_key = 'deleted';
    foreach ($this->condition->conditions() as $condition) {
      if ($condition['field'] === $deleted_key) {
        return $this;
      }
    }

    if (!$this->getMetaData('trash')) {
      $this->addMetaData('trash', $trash_manager->getTrashContext());
    }

    if ($this->getMetaData('trash') === 'inactive') {
      $this->exists($deleted_key);
    }
    else {
      $this->notExists($deleted_key);
    }

    return $this;
  }

}
